var/game/GameController

game
	var/tmp
		var/XML/Element/_root
		list/_units
		list/_dark_age_spells

	proc
		Root()
			if (!_root)
				// Root element for all the xml files found in the demo directory.
				_root = xmlRootFromPath("demo")
			return _root

		UnitList(mob/player)
			// Show the player a list of units they can choose from.
			var/page = "<head><title>Choose your unit</title></head>"

			var/list/units = Units()
			for (var/name in units)
				var/unit/current_unit = units[name]
				page += "<b>[name]</b> - [current_unit.description()] &nbsp;&nbsp;<b><a href=?src=\ref[current_unit];command=choose>\[choose]</a> <a href=?src=\ref[current_unit];command=display>\[stats]</a></b><br>"

			page += "<br><blockquote><i>You can add or edit a unit by tweaking the .xml files in the demo/units directory. On startup, the game reads the .xml files to determine what units are available, what their stats are, and what icon to use. You can therefore allow the player to customize the game, or you can use the XML for easy game creation, and access the files through the cache so the player can't get to them.</i></blockquote>"
			player << browse(page, "window=units;size=600x600;can_close=0")

		Units()
			if (!_units)
				// Haven't loaded them yet.
				_units = new()

				var/XML/Element/root = Root()

				// Get every element called "unit" in the XML and create an object for it.
				for (var/XML/Element/unit_element in root.Descendants("unit"))
					var/unit/unit_object = new(unit_element)
					var/name = unit_object.name()

					_units[name] = unit_object
			return _units

		DarkAgeSpellList(mob/player)
			var/page = "<head><title>Dark Age of Camelot spells</title></head>"
			page += "<i>The spell data is a subset of an XML file provided by Dark Age of Camelot:<br>http://camelotherald.com/xml/</i><br><br>"

			var/XML/Element/root = Root()

			// Dark Age spells are in a spell_list element, so find that.
			var/XML/Element/spell_list = root.FirstChild("spell_list")

			// The spells are grouped into spell_lines, so start by giving them a choice of line.
			page += "<i>Choose a spell line:</i><br>"
			for (var/XML/Element/line in spell_list.Children("spell_line"))
				var/line_name = line.Attribute("name")
				var/element_id = line.Attribute("id")
				page += "<b><a href=?src=\ref[src];command=display_spell_line;id=[element_id]>[line_name]</a></b><br><br>"
			player << browse(page, "window=spells;size=600x600")

		DarkAgeSpellLine(mob/player, line_id)
			var/page = "<head><title>Dark Age of Camelot spells</title></head>"
			page += "<i>Choose a spell:</i><br><br>"

			var/XML/Element/root = Root()

			// This time find things using Descendants(), which keeps us from having to
			// walk the tree looking for specific children. For large XML documents,
			// this might be slower than the approach in DarkAgeSpellList().
			for (var/XML/Element/spell_line in root.Descendants("spell_line"))
				if (spell_line.Attribute("id") == line_id)
					for (var/XML/Element/spell in spell_line.Children("spell"))
						var/spell_name = spell.Attribute("name")
						var/desc = spell.Attribute("desc")
						var/element_id = spell.Attribute("id")
						page += "<b><a href=?src=\ref[src];command=display_spell;id=[element_id]>[spell_name]</a></b><br>&nbsp;&nbsp;&nbsp;&nbsp;[desc]<br><br>"
					break
			page += "<br><center><b><a href=?src=\ref[src];command=display_spell_list>\[Main Spells page]</a></b></center>"
			player << browse(page, "window=spells;size=600x600")

		DarkAgeSpell(mob/player, spell_id)
			var/page = "<head><title>Dark Age of Camelot spells</title></head>"

			var/XML/Element/root = Root()

			// This time around, list the data by just iterating through attributes and children.
			for (var/XML/Element/spell in root.Descendants("spell"))
				if (spell.Attribute("id") == spell_id)
					// First list the attributes except for id
					var/list/attributes = spell.Attributes()
					for (var/attribute in attributes)
						if (attribute != "id")
							page += "<b>[attribute]:</b> [attributes[attribute]]<br>"
					page += "<br>"

					// Now the children. Use ChildElements to make sure we don't get white space and such.
					for (var/XML/Element/child in spell.ChildElements())
						page += "<b>[child.Tag()]:</b> [child.Text()]<br>"

					page += "<br><br>The XML for this spell:<br><pre>[html_encode(spell.XML())]</pre>"
					break
			page += "<br><center><b><a href=?src=\ref[src];command=display_spell_list>\[Main Spells page]</a></b></center>"
			player << browse(page, "window=spells;size=600x600")

		DarkAgeServerList(mob/player)
			var/page = "<head><title>Dark Age of Camelot servers</title></head>"
			page += "<i>The server data is a subset of an XML file provided hourly by Dark Age of Camelot:<br>http://camelotherald.com/xml/</i><br><br>"

			var/XML/Element/root = Root()

			// Dark Age spells are in a server_status element, so find that.
			var/XML/Element/server_status = root.FirstChild("server_status")

			// The spells are grouped into spell_lines, so start by giving them a choice of line.
			page += "<i>Choose a server:</i><br>"
			for (var/XML/Element/server in server_status.Children("server"))
				var/server_name = server.Attribute("name")
				var/population = server.FirstChildText("population")
				var/status = server.FirstChildText("status")
				page += "<b><a href=?src=\ref[src];command=display_server;name=[server_name]>[server_name]</a></b> - [population] players - [status]<br><br>"
			player << browse(page, "window=servers;size=600x600")

		DarkAgeServer(mob/player, server_name)
			var/page = "<head><title>Dark Age of Camelot server</title></head>"
			page += "<h3>[server_name]</h3>"

			var/XML/Element/root = Root()
			var/XML/Element/server_status = root.FirstChild("server_status")

			// This time around, list the data by just iterating through attributes and children.
			for (var/XML/Element/server in server_status.Children("server"))
				if (server.Attribute("name") == server_name)
					// What realm owns which relic right now?
					page += "<b><i>Relic ownership:</i></b><br>"

					for (var/XML/Element/relic in server.Children("relic"))
						page += "<b>[relic.Attribute("name")]</b> - \[Bonus: [relic.Attribute("type")]] - \[Belongs to: [relic.Attribute("realm")]]<br>"

						// Has the relic been stolen?
						var/owner = relic.FirstChildText("owner")
						if (owner != relic.Attribute("realm"))
							page += "&nbsp;&nbsp;&nbsp;&nbsp;<font color=red>Stolen by [owner]!</font><br>"
						page += "<br>"

					// Who owns the keeps?
					page += "<b><i>Keep ownership:</i></b><br>"

					for (var/XML/Element/keep in server.Children("keep"))
						page += "<b>[keep.Attribute("name")]</b> - \[Belongs to: [keep.Attribute("realm")]]<br>"

						// Has the keep been stolen?
						var/owner = keep.FirstChildText("owner")
						if (owner != keep.Attribute("realm"))
							page += "&nbsp;&nbsp;&nbsp;&nbsp;<font color=red>Inhabited by [owner]!</font><br>"

						// Is a guild in charge?
						var/guild = keep.FirstChildText("claimant")
						if (guild)
							page += "&nbsp;&nbsp;&nbsp;&nbsp;<font color=green>Claimed by guild: [guild]!</font><br>"
						page += "<br>"
					break
			page += "<br><center><b><a href=?src=\ref[src];command=display_server_list>\[Main Server page]</a></b></center>"
			player << browse(page, "window=servers;size=600x600")

	Topic(href, href_list[])
		switch(href_list["command"])
			if ("units")
				UnitList(usr)

			if ("display_spell_list")
				DarkAgeSpellList(usr)

			if ("display_spell_line")
				var/line_id = href_list["id"]
				DarkAgeSpellLine(usr, line_id)

			if ("display_spell")
				var/spell_id = href_list["id"]
				DarkAgeSpell(usr, spell_id)

			if ("display_server_list")
				DarkAgeServerList(usr)

			if ("display_server")
				var/server_name = href_list["name"]
				DarkAgeServer(usr, server_name)
unit
	var/tmp
		XML/Element/element

	New(my_element)
		element = my_element
		return ..()

	proc
		name()
			return element.FirstChildText("name") || "Unnamed"

		description()
			return element.FirstChildText("description") || "No description"

		damage()
			return text2num(element.FirstChildText("damage") || "1")

		distance()
			return text2num(element.FirstChildText("distance") || "1")

		picture()
			return element.FirstChildText("image") || "demo/units/generic.bmp"

		Display(mob/player)
			// Iterate through the XML element and display all my data.
			var/page = "<head><title>Unit data</title></head>"
			page += "<h3>[name()]</h3>"
			page += "<table>"
			for (var/XML/Element/child in element.ChildElements())
				page += "<tr><td><b>[child.Tag()]</b></td><td>[child.Text()]</td></tr>"
			page += "</table>"

			// For demo purposes, show the source XML for this.
			page += "<br><i>The XML used to generate this unit:</i><br><pre>[html_encode(element.XML())]</pre>"

			page += "<br><center><a href=?src=\ref[GameController];command=units>\[complete unit list]</a></center>"
			player << browse(page, "window=units")

	Topic(href, href_list[])
		switch(href_list["command"])
			if ("display")
				Display(usr)
			if ("choose")
				usr.SetUnit(src)
				usr << browse(null, "window=units")

turf
	icon = 'demo/ground.dmi'

world
	New()
		GameController = new()
		return ..()

mob
	var/tmp/unit/my_unit

	Login()
		spawn(5)
			GameController.UnitList(src)
		loc = locate(5,5,1)
		return ..()

	verb/debug()
		dd_debugger.extraControlPanelObjects += GameController
		dd_debugger.ControlPanel()

	verb/Dark_Age_Spells()
		GameController.DarkAgeSpellList(src)

	verb/Dark_Age_Servers()
		GameController.DarkAgeServerList(src)

	verb/Select_Unit()
		GameController.UnitList(src)

	proc/SetUnit(unit/a_unit)
		my_unit = a_unit
		var/icon/I = new(my_unit.picture())
		icon = I

	Stat()
		if (my_unit)
			stat("damage", my_unit.damage())
			stat("distance", my_unit.distance())

